@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@ import dbus
 import dbus.glib
 import dbus.service
 import gobject
-import pynotify
 import time
 import locale
 import gettext
@@ -41,17 +40,26 @@ except locale.Error, e:
     os.environ['LC_ALL'] = 'C'
     locale.setlocale (locale.LC_ALL, "")
 
-APPDIR="/usr/share/system-config-printer"
-DOMAIN="system-config-printer"
-ICON="printer"
-SEARCHING_ICON="document-print-preview"
+try:
+    import pynotify
+except RuntimeError, e:
+    print "%s:" % DOMAIN, e
+    print "This is a graphical application and requires DISPLAY to be set."
+    sys.exit (1)
 
-# Let gobject know we'll be using threads.
-gobject.threads_init ()
+APPDIR="/usr/share/system-config-printer"
+ICON="printer"
 
 # We need to call pynotify.init before we can check the server for caps
 pynotify.init('System Config Printer Notification')
 
+# D-Bus APIs of other objects we'll use.
+PRINTING_BUS="org.fedoraproject.Config.Printing"
+PRINTING_PATH="/org/fedoraproject/Config/Printing"
+PRINTING_IFACE="org.fedoraproject.Config.Printing"
+NEWPRINTERDIALOG_IFACE=PRINTING_IFACE + ".NewPrinterDialog"
+PRINTERPROPERTIESDIALOG_IFACE=PRINTING_IFACE + ".PrinterPropertiesDialog"
+
 ####
 #### NewPrinterNotification DBus server (the 'new' way).
 ####
@@ -64,49 +72,45 @@ class NewPrinterNotification(dbus.service.Object):
     STATUS_GENERIC_DRIVER = 2
     STATUS_NO_DRIVER = 3
 
-    def __init__ (self, bus):
-        self.bus = bus
+    def __init__ (self, system_bus, session_bus):
+        self.system_bus = system_bus
+        self.session_bus = session_bus
         self.getting_ready = 0
-        bus_name = dbus.service.BusName (PDS_OBJ, bus=bus)
+        bus_name = dbus.service.BusName (PDS_OBJ, bus=system_bus)
         dbus.service.Object.__init__ (self, bus_name, PDS_PATH)
-
-    def wake_up (self):
-        global waitloop, runloop, viewer
-        import jobviewer
-        if viewer == None:
-            try:
-                waitloop.quit ()
-            except:
-                pass
-            runloop = gobject.MainLoop ()
-            viewer = jobviewer.JobViewer(bus=bus, loop=runloop,
-                                         trayicon=trayicon,
-                                         suppress_icon_hide=True)
+        self.notification = None
 
     @dbus.service.method(PDS_IFACE, in_signature='', out_signature='')
     def GetReady (self):
-        self.wake_up ()
+        TIMEOUT=1200000
         if self.getting_ready == 0:
-            viewer.set_special_statusicon (SEARCHING_ICON,
-                                           tooltip=_("Configuring new printer"))
+            n = pynotify.Notification (_("Configuring new printer"),
+                                       _("Please wait..."),
+                                       'printer')
+            n.set_timeout (TIMEOUT + 5000)
+            n.set_data ('closed', False)
+            n.connect ('closed', self.on_notification_closed)
+            n.show ()
+            self.notification = n
 
         self.getting_ready += 1
-        gobject.timeout_add_seconds (1200, self.timeout_ready)
+        gobject.timeout_add_seconds (TIMEOUT, self.timeout_ready)
+
+    def on_notification_closed (self, notification):
+        notification.set_data ('closed', True)
 
     def timeout_ready (self):
-        global viewer
         if self.getting_ready > 0:
             self.getting_ready -= 1
-        if self.getting_ready == 0:
-            viewer.unset_special_statusicon ()
+        if (self.getting_ready == 0 and
+            self.notification and
+            not self.notification.get_data ('closed')):
+            self.notification.close ()
 
         return False
 
     @dbus.service.method(PDS_IFACE, in_signature='isssss', out_signature='')
     def NewPrinter (self, status, name, mfg, mdl, des, cmd):
-        global viewer
-        self.wake_up ()
-
         if name.find("/") >= 0:
             # name is a URI, no queue was generated, because no suitable
             # driver was found
@@ -128,8 +132,7 @@ class NewPrinterNotification(dbus.service.Object):
                               lambda x, y:
                                   self.setup_printer (x, y, name, devid))
             else:
-                args = ["--setup-printer", name, "--devid", devid]
-                self.run_config_tool (args)
+                self.setup_printer (None, None, name, devid)
 
         else:
             # name is the name of the queue which hal_lpadmin has set up
@@ -198,7 +201,7 @@ class NewPrinterNotification(dbus.service.Object):
                     n.set_urgency (pynotify.URGENCY_NORMAL)
                     n.add_action ("test-page", _("Print test page"),
                                   lambda x, y:
-                                      self.print_test_page (x, y, name, devid))
+                                      self.print_test_page (x, y, name))
                     n.add_action ("configure", _("Configure"),
                                   lambda x, y: self.configure (x, y, name))
             else: # Model mismatch
@@ -216,45 +219,51 @@ class NewPrinterNotification(dbus.service.Object):
                                   self.find_driver (x, y, name, devid))
                     n.set_timeout (pynotify.EXPIRES_NEVER)
                 else:
-                    self.run_config_tool (["--configure-printer",
-                                           name, "--no-focus-on-map"])
+                    self.configure (None, None, name)
 
-        viewer.notify_new_printer (name, n)
-        # Set the icon back how it was.
         self.timeout_ready ()
+        n.show ()
+        self.notification = n
 
-    def run_config_tool (self, argv):
-        import os
-        pid = os.fork ()
-        if pid == 0:
-            # Child.
-            cmd = "/usr/bin/system-config-printer"
-            argv.insert (0, cmd)
-            os.execvp (cmd, argv)
-            sys.exit (1)
-        elif pid == -1:
-            print "Error forking process"
-        else:
-            gobject.timeout_add_seconds (60, self.collect_exit_code, pid)
-
-    def print_test_page (self, notification, action, name, devid = ""):
-        args = ["--print-test-page", name]
-        if devid != "":
-            args.extend (["--devid", devid])
-        self.run_config_tool (args)
+    def print_test_page (self, notification, action, name):
+        path = self.configure (None, None, name)
+        obj = self.session_bus.get_object (PRINTING_BUS, path)
+        iface = dbus.Interface (obj, PRINTERPROPERTIESDIALOG_IFACE)
+        iface.PrintTestPage ()
 
     def configure (self, notification, action, name):
-        self.run_config_tool (["--configure-printer", name])
+        obj = self.session_bus.get_object (PRINTING_BUS, PRINTING_PATH)
+        iface = dbus.Interface (obj, PRINTING_IFACE)
+        return iface.PrinterPropertiesDialog (dbus.UInt32(0), name)
+
+    def get_newprinterdialog_interface (self):
+        obj = self.session_bus.get_object (PRINTING_BUS, PRINTING_PATH)
+        iface = dbus.Interface (obj, PRINTING_IFACE)
+        path = iface.NewPrinterDialog ()
+        obj = self.session_bus.get_object (PRINTING_BUS, path)
+        iface = dbus.Interface (obj, NEWPRINTERDIALOG_IFACE)
+        return iface
+
+    def ignore_dbus_replies (self, *args):
+        pass
 
     def find_driver (self, notification, action, name, devid = ""):
-        args = ["--choose-driver", name]
-        if devid != "": args = args + ["--devid", devid]
-        self.run_config_tool (args)
+        try:
+            iface = self.get_newprinterdialog_interface ()
+            iface.ChangePPD (dbus.UInt32(0), name, devid,
+                             reply_handler=self.ignore_dbus_replies,
+                             error_handler=self.ignore_dbus_replies)
+        except dbus.DBusException:
+            pass
 
     def setup_printer (self, notification, action, uri, devid = ""):
-        args = ["--setup-printer", uri]
-        if devid != "": args = args + ["--devid", devid]
-        self.run_config_tool (args)
+        try:
+            iface = self.get_newprinterdialog_interface ()
+            iface.NewPrinterFromDevice (dbus.UInt32(0), uri, devid,
+                                        reply_handler=self.ignore_dbus_replies,
+                                        error_handler=self.ignore_dbus_replies)
+        except dbus.DBusException:
+            pass
 
     def install_driver (self, notification, action, missing_pkgs):
         try:
@@ -280,7 +289,7 @@ class NewPrinterNotification(dbus.service.Object):
 
 PROGRAM_NAME="system-config-printer-applet"
 def show_help ():
-    print "usage: %s [--no-tray-icon]" % PROGRAM_NAME
+    print "usage: %s [--help|--version|--debug]" % PROGRAM_NAME
 
 def show_version ():
     import config
@@ -290,18 +299,81 @@ def show_version ():
 #### Main program entry
 ####
 
-global waitloop, runloop, viewer
+def monitor_session (*args):
+    pass
 
-trayicon = True
-waitloop = runloop = None
-viewer = None
+def any_jobs ():
+    try:
+        c = cups.Connection ()
+        jobs = c.getJobs (my_jobs=True, limit=1)
+        if len (jobs):
+            return True
+    except:
+        pass
+
+    return False
+
+class RunLoop:
+    DBUS_PATH="/com/redhat/PrinterSpooler"
+    DBUS_IFACE="com.redhat.PrinterSpooler"
+
+    def __init__ (self, session_bus, system_bus, loop):
+        self.system_bus = system_bus
+        self.session_bus = session_bus
+        self.loop = loop
+        self.timer = None
+        system_bus.add_signal_receiver (self.handle_dbus_signal,
+                                        path=self.DBUS_PATH,
+                                        dbus_interface=self.DBUS_IFACE)
+        self.check_for_jobs ()
+
+    def remove_signal_receiver (self):
+        self.system_bus.remove_signal_receiver (self.handle_dbus_signal,
+                                                path=self.DBUS_PATH,
+                                                dbus_interface=self.DBUS_IFACE)
+
+    def run (self):
+        self.loop.run ()
+
+    def __del__ (self):
+        self.remove_signal_receiver ()
+        if self.timer:
+            gobject.source_remove (self.timer)
+
+    def handle_dbus_signal (self, *args):
+        if self.timer:
+            gobject.source_remove (self.timer)
+        self.timer = gobject.timeout_add (200, self.check_for_jobs)
+
+    def check_for_jobs (self, *args):
+        debugprint ("checking for jobs")
+        if any_jobs ():
+            if self.timer != None:
+                gobject.source_remove (self.timer)
+
+            self.remove_signal_receiver ()
+
+            # Start the job applet.
+            debugprint ("Starting job applet")
+            try:
+                obj = self.session_bus.get_object (PRINTING_BUS, PRINTING_PATH)
+                iface = dbus.Interface (obj, PRINTING_IFACE)
+                path = iface.JobApplet ()
+                debugprint ("Job applet is %s" % path)
+            except dbus.DBusException, e:
+                try:
+                    print e
+                except:
+                    pass
+
+        # Don't run this timer again.
+        return False
 
 if __name__ == '__main__':
     import sys, getopt
     try:
         opts, args = getopt.gnu_getopt (sys.argv[1:], '',
-                                        ['no-tray-icon',
-                                         'debug',
+                                        ['debug',
                                          'help',
                                          'version'])
     except getopt.GetoptError:
@@ -315,8 +387,6 @@ if __name__ == '__main__':
         if opt == "--version":
             show_version ()
             sys.exit (0)
-        if opt == "--no-tray-icon":
-            trayicon = False
         elif opt == "--debug":
             set_debugging (True)
 
@@ -328,23 +398,9 @@ if __name__ == '__main__':
         except:
             pass
 
-    if trayicon:
-        # Stop running when the session ends.
-        def monitor_session (*args):
-            pass
-
-        try:
-            bus = dbus.SessionBus()
-            bus.add_signal_receiver (monitor_session)
-        except:
-            try:
-                print >> sys.stderr, ("%s: failed to connect to "
-                                      "session D-Bus" % PROGRAM_NAME)
-            finally:
-                sys.exit (1)
-
+    system_bus = session_bus = None
     try:
-        bus = dbus.SystemBus()
+        system_bus = dbus.SystemBus()
     except:
         try:
             print >> sys.stderr, ("%s: failed to connect to system D-Bus" %
@@ -352,96 +408,41 @@ if __name__ == '__main__':
         finally:
             sys.exit (1)
 
-    if trayicon:
+    try:
+        session_bus = dbus.SessionBus()
+        # Stop running when the session ends.
+        session_bus.add_signal_receiver (monitor_session)
+    except:
         try:
-            NewPrinterNotification(bus)
-        except:
-            try:
-                print >> sys.stderr, ("%s: failed to start "
-                                      "NewPrinterNotification service" %
-                                      PROGRAM_NAME)
-            except:
-                pass
-
-        try:
-            cupshelpers.installdriver.set_debugprint_fn (debugprint)
-            cupshelpers.installdriver.PrinterDriversInstaller(bus)
-        except Exception, e:
-            try:
-                print >> sys.stderr, ("%s: failed to start "
-                                      "PrinterDriversInstaller service: %s" %
-                                      (PROGRAM_NAME, e))
-                pass
-            except:
-                pass
-
-    if trayicon and get_debugging () == False:
-        # Start off just waiting for print jobs.
-        def any_jobs ():
-            try:
-                c = cups.Connection ()
-                jobs = c.getJobs (my_jobs=True, limit=1)
-                if len (jobs):
-                    return True
-            except:
-                pass
-
-            return False
-
-        if not any_jobs ():
-
-            ###
-            class WaitForJobs:
-                DBUS_PATH="/com/redhat/PrinterSpooler"
-                DBUS_IFACE="com.redhat.PrinterSpooler"
-
-                def __init__ (self, bus, waitloop):
-                    self.bus = bus
-                    self.waitloop = waitloop
-                    self.timer = None
-                    bus.add_signal_receiver (self.handle_dbus_signal,
-                                             path=self.DBUS_PATH,
-                                             dbus_interface=self.DBUS_IFACE)
-
-                def __del__ (self):
-                    bus = self.bus
-                    bus.remove_signal_receiver (self.handle_dbus_signal,
-                                                path=self.DBUS_PATH,
-                                                dbus_interface=self.DBUS_IFACE)
-                    if self.timer:
-                        gobject.source_remove (self.timer)
-
-                def handle_dbus_signal (self, *args):
-                    if self.timer:
-                        gobject.source_remove (self.timer)
-                    self.timer = gobject.timeout_add (200, self.check_for_jobs)
-
-                def check_for_jobs (self, *args):
-                    debugprint ("checking for jobs")
-                    if any_jobs ():
-                        gobject.source_remove (self.timer)
-                        self.waitloop.quit ()
-
-                    # Don't run this timer again.
-                    return False
-            ###
-
-            waitloop = gobject.MainLoop ()
-            jobwaiter = WaitForJobs(bus, waitloop)
-            waitloop.run()
-            del jobwaiter
-            waitloop = None
-
-    if viewer == None:
-        import jobviewer
-        import gtk
-        runloop = gobject.MainLoop ()
-        gtk.window_set_default_icon_name ('printer')
-        viewer = jobviewer.JobViewer(bus=bus, loop=runloop,
-                                     trayicon=trayicon)
+            print >> sys.stderr, ("%s: failed to connect to "
+                                  "session D-Bus" % PROGRAM_NAME)
+        finally:
+            sys.exit (1)
 
     try:
-        runloop.run()
+        NewPrinterNotification(system_bus, session_bus)
+    except:
+        try:
+            print >> sys.stderr, ("%s: failed to start "
+                                  "NewPrinterNotification service" %
+                                  PROGRAM_NAME)
+        except:
+            pass
+
+    try:
+        cupshelpers.installdriver.set_debugprint_fn (debugprint)
+        cupshelpers.installdriver.PrinterDriversInstaller(system_bus)
+    except Exception, e:
+        try:
+            print >> sys.stderr, ("%s: failed to start "
+                                  "PrinterDriversInstaller service: "
+                                  "%s" % (PROGRAM_NAME, e))
+        except:
+            pass
+
+    loop = gobject.MainLoop ()
+    runloop = RunLoop (session_bus, system_bus, loop)
+    try:
+        runloop.run ()
     except KeyboardInterrupt:
         pass
-    viewer.cleanup ()
