@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 
 ## system-config-printer
 
@@ -20,18 +20,15 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import dbus
+import gobject
 import gtk
 import cupshelpers
 
 import asyncconn
 from debug import debugprint
+from gettext import gettext as _
 
-_ = lambda x: x
-def set_gettext_function (fn):
-    global _
-    _ = fn
-
-class PPDsLoader:
+class PPDsLoader(gobject.GObject):
     """
     1. If PackageKit support is available, and this is a local server,
     try to use PackageKit to install relevant drivers.  We do this
@@ -46,12 +43,19 @@ class PPDsLoader:
     4. If Jockey was able to install one, fetch the list of available
     drivers again.
     """
-    def __init__ (self, callback, device_id=None, device_uri=None, parent=None,
-                  host=None, encryption=None, language=None):
+
+    __gsignals__ = {
+        'finished': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        }
+
+    def __init__ (self, device_id=None, parent=None, device_uri=None,
+                  host=None, encryption=None, language=None,
+                  device_make_and_model=None):
+        gobject.GObject.__init__ (self)
         debugprint ("+%s" % self)
-        self._callback = callback
         self._device_id = device_id
         self._device_uri = device_uri
+        self._device_make_and_model = device_make_and_model
         self._parent = parent
         self._host = host
         self._encryption = encryption
@@ -60,7 +64,9 @@ class PPDsLoader:
         self._installed_files = []
         self._conn = None
         self._ppds = None
-        self._ppdsmatch_result = (cupshelpers.ppds.PPDs.STATUS_NO_DRIVER, '')
+        self._exc = None
+
+        self._ppdsmatch_result = None
         self._jockey_queried = False
         self._local_cups = (self._host == None or
                             self._host == "localhost" or
@@ -83,6 +89,7 @@ class PPDsLoader:
 
         self._dialog.connect ("response", self._dialog_response)
 
+    def run (self):
         self._dialog.show_all ()
         if self._local_cups and self._device_id and self._bus:
             self._devid_dict = cupshelpers.parseDeviceID (self._device_id)
@@ -101,9 +108,12 @@ class PPDsLoader:
             self._dialog.destroy ()
             self._dialog = None
 
-        self._callback = None
         self._parent = None
 
+        if self._conn:
+            self._conn.destroy ()
+            self._conn = None
+
     def get_installed_files (self):
         return self._installed_files
 
@@ -113,8 +123,11 @@ class PPDsLoader:
     def get_ppdsmatch_result (self):
         return self._ppdsmatch_result
 
+    def get_error (self):
+        return self._exc
+
     def _dialog_response (self, dialog, response):
-        self._call_callback (None)
+        self.emit ('finished')
 
     def _query_cups (self):
         debugprint ("Asking CUPS for PPDs")
@@ -128,34 +141,38 @@ class PPDsLoader:
             self._cups_connect_reply(self._conn, None)
 
     def _cups_connect_reply (self, conn, UNUSED):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
         conn._begin_operation (_("fetching PPDs"))
-        conn.getPPDs (reply_handler=self._cups_reply,
-                      error_handler=self._cups_error)
+        try:
+            conn.getPPDs2 (reply_handler=self._cups_reply,
+                           error_handler=self._cups_error)
+        except AttributeError:
+            # getPPDs requires pycups >= 1.9.52
+            conn.getPPDs (reply_handler=self._cups_reply,
+                          error_handler=self._cups_error)
 
     def _cups_reply (self, conn, result):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
         ppds = cupshelpers.ppds.PPDs (result, language=self._language)
         self._ppds = ppds
         self._need_requery_cups = False
         if self._device_id:
-            self._ppdsmatch_result = ppds.\
-                getPPDNameFromDeviceID (self._devid_dict["MFG"],
-                                        self._devid_dict["MDL"],
-                                        self._devid_dict["DES"],
-                                        self._devid_dict["CMD"],
-                                        self._device_uri,
-                                        self._installed_files)
+            fit = ppds.\
+                getPPDNamesFromDeviceID (self._devid_dict["MFG"],
+                                         self._devid_dict["MDL"],
+                                         self._devid_dict["DES"],
+                                         self._devid_dict["CMD"],
+                                         self._device_uri,
+                                         self._device_make_and_model)
 
-            (status, ppdname) = self._ppdsmatch_result
+            ppdnamelist = ppds.\
+                orderPPDNamesByPreference (fit.keys (),
+                                           self._installed_files,
+                                           devid=self._devid_dict,
+                                           fit=fit)
+            self._ppdsmatch_result = (fit, ppdnamelist)
+
+            ppdname = ppdnamelist[0]
             if (self._bus and
-                status != ppds.STATUS_SUCCESS and
+                not fit[ppdname].startswith ("exact") and
                 not self._jockey_queried and
                 self._local_cups):
                 # Try to install packages using jockey if
@@ -167,17 +184,13 @@ class PPDsLoader:
 
         conn.destroy ()
         self._conn = None
-        self._call_callback (None)
+        self.emit ('finished')
 
     def _cups_error (self, conn, exc):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
         conn.destroy ()
         self._conn = None
         self._ppds = None
-        self._call_callback (exc)
+        self.emit ('finished')
 
     def _query_packagekit (self):
         debugprint ("Asking PackageKit to install drivers")
@@ -245,12 +258,9 @@ class PPDsLoader:
         else:
             self._conn.destroy ()
             self._conn = None
-            self._call_callback (None)
-
-    def _call_callback (self, exc):
-        if self._callback:
-            self._callback (self, exc)
+            self.emit ('finished')
 
+gobject.type_register(PPDsLoader)
 
 if __name__ == "__main__":
     class Foo:
@@ -264,12 +274,15 @@ if __name__ == "__main__":
             self._window = w
 
         def go (self, button):
-            PPDsLoader (self.ppds_loaded, device_id="MFG:MFG;MDL:MDL;",
-                        parent=self._window)
+            loader = PPDsLoader (device_id="MFG:MFG;MDL:MDL;",
+                                 parent=self._window)
+            loader.connect ('finished', self.ppds_loaded)
+            loader.run ()
 
-        def ppds_loaded (self, ppdsloader, exc):
+        def ppds_loaded (self, ppdsloader):
             self._window.destroy ()
             gtk.main_quit ()
+            exc = ppdsloader.get_error ()
             print exc
             ppds = ppdsloader.get_ppds ()
             if ppds != None:
